﻿Type.registerNamespace('ApracJs.UI.Grid');

ApracJs.UI.Grid.GridConfigAction = function() {
    ApracJs.UI.Grid.GridConfigAction.initializeBase(this);
}

ApracJs.UI.Grid.GridConfigAction.prototype = {
    initialize: function() {
        ApracJs.UI.Grid.GridConfigAction.callBaseMethod(this, 'initialize');
        var gd = this.get_grid();
        this._allowFrozen = gd.get_allowFrozen() && gd.get_showHeader();
        this._frozenColumnCount = gd.get_frozenColumnCount();
        this._maxLevel = gd.get_maxHeaderLevel();

        if (!gd) throw Error.argumentNull("_grid");

        this._lockUrl = 'aicon-07';

        this._refresh();
        this._modified = false;
        this._frozenModified = false;

        var form = this.get_form();
        var grid = form.grid;
        var count = grid.get_recordCount();
        if (count > 10) {
            grid.set_allowFilter(true); // 没有pager的情况下起定位作用
            grid.findColumn("designCaption").get_filterControl().set_nullDisplayText("查找定位列");
        } else if (count == 0) {
            $common.showTips(Sys.Res.noColumnConfig);
            form.close();
            return;
        }
        form.btnFrozen.set_visible(this._allowFrozen);
        form.btnFrozen.set_hint('<div class="ListLabel">锁定功能说明:<p class="Label">锁定列默认显示到最前面</p><p class="Label">锁定功能是锁定多少列，是一个数量，不是固定锁定某列</p><p class="Label">移动列位置是交换列的顺序，但不改变原有的锁定数，也就是不改变锁定列状态</p></div>');
        grid.findColumn("lockUrl").set_visible(this._allowFrozen);

        if (this._maxLevel > 1) {
            grid.set_allowDrag(false); // 多表头不支持拖动
            grid.set_allowMultiSelectedRow(false); // 多表头不支持多选拖动
            grid.set_allowDrop(false);
            grid.set_allowCopy(true);

            form.btnDrag.set_visible(false);
            form.set_caption(Sys.Res.noColumnDrag);
            form.btnTop.set_visible(false);
            form.btnDown.set_visible(false);
        }
    },

    dispose: function() {
        var grid = this.get_grid();
        if (this._allowFrozen) {                                            // 是否开启了冻结功能
            if (this._frozenModified) {
                var fcount = this._frozenColumnCount;
                if (fcount <= this._firstVisibleIndex) fcount = 0; // fix bug 25478 前面1列或多列不允许配置，列配置界面无法解锁的问题；
                grid.setFrozenColumnCount(fcount); // 关闭的时候去改变列锁定功能，同时将数据保存到配置文件中
            } else if (this._frozenColumnCount > 0) {
                grid._setFrozenVisible(true);                               // 由于打开列配置界面的时候将原有锁定列隐藏了，关闭窗体时需要还原
                grid._internalFrozenRowsHeight();
            }
        }
        if (this._modified) {
            grid._internalSaveConfig();                                     // 将配置信息保存到服务器
        }
        delete this._columnList;
        ApracJs.UI.Grid.GridConfigAction.callBaseMethod(this, 'dispose');
    },

    get_grid: function() {
        return this.get_form()._grid;
    },

    _setModified: function() {
        this._modified = true;
    },

    _refresh: function() {
        delete this._columnList;

        var grid = this.get_grid();
        var hideNotAllowConfigColumns = this.get_form()._hideNotAllowConfigColumns;
        var count = grid.get_columnCount();

        this._maxFrozenIndex = 0;
        var columnList = this._columnList = [];
        for (var i = 0, n = 0; i < count; i++) {
            var column = grid.getColumnByIndex(i);
            var visible = column.get_visible();
            var allowConfig = column.get_allowConfig();
            var isExpand = column._isExpand; // 扩展列不允许配置，不然会有很难处理的bug

            if (hideNotAllowConfigColumns && !allowConfig) {
                continue;
            }
            if (Object.isUndefinedOrNull(this._firstVisibleIndex)) {
                this._firstVisibleIndex = i;
            }
            var lockUrl = "";
            var frozenIndex = column.get_frozenIndex();
            this._maxFrozenIndex = Math.max(frozenIndex, this._maxFrozenIndex);
            if (frozenIndex < this._frozenColumnCount) {
                lockUrl = this._lockUrl;
            }
            //if (lockUrl != "" && !visible) continue; // 隐藏的冻结列不支持配置

            var _bandCaption = "";
            if (column.get_band()) {
                _bandCaption = column.get_band().caption;
                if (_bandCaption != "") _bandCaption += "-";
            }

            var record = {};
            record.allowFrozen = column.get_allowFrozen();
            record.allowConfig = allowConfig;
            record.name = column.get_name();
            record.oldCaption = column.get_designCaption();
            record.designCaption = _bandCaption + column.get_designCaption() || '(空)';
            record.caption = column.get_caption();
            record.visible = visible;
            record.bandIndex = column.get_bandIndex();
            record.isExpand = isExpand;
            record.lockUrl = lockUrl;
            record.frozenIndex = frozenIndex;
            record.columnIndex = i;
            columnList[n++] = record;
        }

        var configGrid = this.get_form().grid;
        configGrid.refresh(columnList);

        for (var i = 0, count = columnList.length; i < count; i++) {
            var record = columnList[i];
            var hint = record.allowConfig ? "" : $craba.getLangPro("本列不允许配置");
            configGrid.setRowHint(i, hint);
        }
    },

    doToggleVisible: function (sender) {
        var columnList = this._columnList;
        var configGrid = sender.get_form().grid;
        var sData = configGrid.get_selectedRowData();
        if (!sData.allowConfig) return;

        this._setModifyForzen(sData);
        var visible = !sData.visible;
        if (!visible) {
            var visibleCount = 0;
            var columns = this.get_grid().get_columns();
            for (var i = 0, count = columns.length; i < count; i++) {
                var c = columns[i];
                if (c.get_visible() && c.get_allowConfig()) { // 不允许配置的列不参与计数
                    visibleCount++;
                }
                if (visibleCount > 2) break;
            }
            if (visibleCount <= 2) {
                $common.showTips($craba.getLangPro("不允许隐藏")); // 有时候（部分AllowConfig=false)提示至少显示2列容易误导用户
                return;
            }

            var bandIndex = sData.bandIndex;
            var bandVisibleCount = 0;
            for (var i = 0, count = columnList.length; i < count; i++) {
                var columnData = columnList[i];
                if (bandIndex === -1 || columnData.bandIndex == bandIndex && columnData.visible) {
                    bandVisibleCount++;
                }
            }
            if (bandVisibleCount <= 1) {
                $common.showTips(Sys.Res.mustColumn);
                return;
            }
        }
        sData.visible = visible; // 做修改
        configGrid.modifySelectedRowData(sData);

        var grid = this.get_grid();
        grid._setFrozenVisible(true);
        grid.getColumn(sData.name).set_visible(visible, true);
        grid._setFrozenVisible(false);
        this.doSelectionChanged(configGrid);
        this._setModified();
    },

    doModifyCaption: function(sender) {
        var configGrid = sender.get_form().grid;
        var sData = configGrid.get_selectedRowData();
        if (!sData.allowConfig) return;

        var tips = Sys.Res.inputColumnCaption;
        if (sData.caption != sData.oldCaption) tips = $craba.getLangPros('原标题', '【' + sData.oldCaption + '】，', tips);
        $msg.prompt(tips, {
            title: $craba.getLangPro('修改列标题'),
            value: sData.caption,
            iconVisible: false,
            maxLength: 10,
            handler: this._doSaveCaption
        }, this);
    },

    _doSaveCaption: function(result) {
        if (result == null) return;
        if (result == '') {
            $common.showInfo($craba.getLangPro('值不能为空'));
            return;
        }

        var configGrid = this.get_form().grid;
        var sData = configGrid.get_selectedRowData();
        var caption = sData.caption;
        var caption2 = result;
        if (caption != caption2 && caption2 != '') {
            sData.caption = caption2;
            configGrid.modifySelectedRowData(sData);
            this.get_grid().getColumn(sData.name).set_caption(caption2, true);
            this._setModified();
        }
    },

    doCellRendering: function(sender, eventArgs) {
        var record = sender.getRowData(eventArgs.get_rowIndex());
        if (this._frozenColumnCount > 0 && record.frozenIndex < this._frozenColumnCount) {
            record.lockUrl = this._lockUrl;
        } else {
            record.lockUrl = "";
        }
        if (!record.allowConfig) {
            eventArgs.set_fontColor("#ACA899");
            eventArgs.set_bgColor($skin.getDisabledBackgroundColor());
            return;
        }
        var bandIndex = record.bandIndex;
        if (bandIndex >= 0) {
            if (!this._rowColors) {
                this._rowColors = ["#EFFFE5", "#FFF6E5", "#FFF4FB", "#F0FFFB", "#FDFFF7"];
            }
            eventArgs.set_bgColor(this._rowColors[bandIndex % 5]);
        }
    },

    doSelectionChanged: function(sender) {
        var form = sender.get_form();
        var configGrid = form.grid;
        if (configGrid.get_recordCount() < 1) return;

        var sData = configGrid.get_selectedRowData();
        if (!sData) {
            form.btnMoveUp.set_enabled(false);
            form.btnTop.set_enabled(false);
            form.btnMoveDown.set_enabled(false);
            form.btnDown.set_enabled(false);
            form.btnVisible.set_enabled(false);
            form.btnFrozen.set_enabled(false);
            form.btnCaption.set_enabled(false);
            return;
        }
        var allowConfig = sData.allowConfig;
        var allowFrozen = sData.allowFrozen;
        var isExpand = sData.isExpand;
        form.btnCaption.set_enabled(allowConfig);

        var rowIndex = configGrid.get_selectedRowIndex();
        var curr = sender.getRowData(rowIndex);

        var canMoveUp = rowIndex > 0;
        if (canMoveUp) {
            var prev = sender.getRowData(rowIndex - 1);
            var frozen = prev.frozenIndex == this._frozenColumnCount - 1 && (prev && prev.frozenIndex == curr.forzenIndex);
            canMoveUp = curr.bandIndex == prev.bandIndex && !frozen;
        }
        form.btnMoveUp.set_enabled(canMoveUp && !isExpand);
        form.btnTop.set_enabled(canMoveUp && !isExpand);

        var canMoveDown = rowIndex < configGrid.get_recordCount() - 1;
        if (canMoveDown) {
            var next = sender.getRowData(rowIndex + 1);
            var frozen = curr.frozenIndex == this._frozenColumnCount - 1 && (prev && prev.frozenIndex == curr.forzenIndex);
            canMoveDown = curr.bandIndex == next.bandIndex && !frozen;
        }
        form.btnMoveDown.set_enabled(canMoveDown && !isExpand);
        form.btnDown.set_enabled(canMoveDown && !isExpand);

        var recordCount = configGrid.get_recordCount() - 1;
        var isFrozen = curr.frozenIndex < this._frozenColumnCount;
        var canFrozen = false;
        if (this._maxLevel == 1) {
            canFrozen = (rowIndex < recordCount || this._frozenColumnCount < curr.frozenIndex);
        } else {
            canFrozen = (rowIndex < recordCount && curr.frozenIndex < this._maxFrozenIndex);
        }

        form.btnVisible.set_enabled(allowConfig);
        form.btnFrozen.set_enabled(canFrozen && this._allowFrozen && allowFrozen);
        form.btnFrozen.set_text($craba.getLangPro(isFrozen ? "解锁" : "锁定"));
    },

    doMoveUp: function(sender) {
        this._doMove(sender, -1);
    },

    doMoveDown: function(sender) {
        this._doMove(sender, 1);
    },

    doTop: function(sender) {
        var grid = this.get_form().grid;
        var dragDatas = grid.createDragDatas();
        if (dragDatas.length <= 1) {
            var rowData = grid.get_selectedRowData();
            if (!rowData) return;
            dragDatas = [rowData];
        }
        this._toMove(0, dragDatas);
    },
    doDown: function(sender) {
        var grid = this.get_form().grid;
        var dragDatas = grid.createDragDatas();
        if (dragDatas.length <= 1) {
            var rowData = grid.get_selectedRowData();
            if (!rowData) return;
            dragDatas = [rowData];
        }
        this._toMove(grid.get_recordCount(), dragDatas);
    },

    _doMove: function(sender, inc, lockUrl) {
        var configGrid = sender.get_form().grid;
        var rowIndex1 = configGrid.get_selectedRowIndex();
        var rowIndex2 = rowIndex1 + inc;
        if (rowIndex2 < 0 || rowIndex2 >= configGrid.get_recordCount()) return;

        var data1 = configGrid.getRowData(rowIndex1);
        var data2 = configGrid.getRowData(rowIndex2);

        var lockUrl1 = data1.lockUrl;
        var lockUrl2 = (typeof (lockUrl) != "undefined") ? lockUrl : data2.lockUrl;
        data1.lockUrl = lockUrl2;
        data2.lockUrl = lockUrl1;

        //如果移动的是锁定列，则标记下来
        this._frozenModified = (this._frozenModified || data1.lockUrl == this._lockUrl || data2.lockUrl == this._lockUrl);

        var frozenIndex1 = data1.frozenIndex;
        data1.frozenIndex = data2.frozenIndex;
        data2.frozenIndex = frozenIndex1;

        configGrid.modifyRowData(rowIndex1, data2);
        configGrid.modifyRowData(rowIndex2, data1);
        configGrid.set_selectedRowIndex(rowIndex2);

        var hint1 = data2.allowConfig ? "" : Sys.Res.noConfig;
        configGrid.setRowHint(rowIndex1, hint1);

        var hint2 = data1.allowConfig ? "" : Sys.Res.noConfig;
        configGrid.setRowHint(rowIndex2, hint2);

        var grid = this.get_grid();
        var column1 = grid.getColumn(data1.name);
        var column2 = grid.getColumn(data2.name);
        grid.clearSortFlag(); //列配置时，清除grid的排序标记，如果一直带着标记，会导致js错误    2015-02-12  ljb
        grid._internalSwapColumn(column1, column2);
        this._setModified();
    },

    doFrozen: function(sender) {
        var configGrid = sender.get_form().grid;
        var grid = this.get_grid();
        var rowIndex = configGrid.get_selectedRowIndex();
        var data = configGrid.getRowData(rowIndex);
        var isFrozen = data.frozenIndex < this._frozenColumnCount; //当前行锁定状态

        //配置窗体关闭的时候列锁定更改才生效，避免频繁的操作
        this._setModified();
        this._frozenModified = true;

        if (this._maxLevel == 1) { //目前，锁定列只支持单表头调整列序；多表头没做列序改变，但支持锁定
            var column1 = grid.getColumn(data.name);
            var lastIndex = this._getLastFrozenIndex();

            if (rowIndex < lastIndex) {
                this._doMove(sender, lastIndex - rowIndex, "");
                lastIndex = this._getLastFrozenIndex();
                data = configGrid.getRowData(lastIndex - 1);
                this._frozenColumnCount = data.frozenIndex + 1;
            } else if (rowIndex > lastIndex + 1) {
                this._doMove(sender, lastIndex - rowIndex + 1, this._lockUrl);
                this._frozenColumnCount = data.frozenIndex + 1;
            } else {
                var c = 0;
                if (isFrozen) {
                    if (rowIndex > 0) {
                        var prevData = configGrid.getRowData(rowIndex - 1);
                        c = prevData.frozenIndex + 1;
                    } else {
                        c = 0;
                    }
                } else {
                    c = data.frozenIndex + 1;
                }
                this._frozenColumnCount = c;

                if (isFrozen) {
                    data.lockUrl = "";
                } else {
                    data.lockUrl = this._lockUrl;
                }
                configGrid.modifyRowData(rowIndex, data);
            }
            this.doSelectionChanged(configGrid);
        } else {
            this._frozenColumnCount = isFrozen ? data.frozenIndex : data.frozenIndex + 1;
            configGrid.refresh();
            configGrid.set_selectedRowIndex(rowIndex);
        }
    },

    _getLastFrozenIndex: function() {
        var configGrid = this.get_form().grid;
        var data = configGrid.get_dataSource();
        var lastIndex = -1;
        for (var i = 0; i < data.length; i++) {
            var row = configGrid.getRowData(i);
            if (row.lockUrl == "") break;
            lastIndex++;
        }
        return lastIndex;
    },

    doReset: function(sender) {
        Sys.UI.MessageBox.confirm(Sys.Res.resetConfig, Function.createDelegate(this, this._doReset));
    },

    _doReset: function(result) {
        if (!result) return;
        var grid = this.get_grid();
        this._frozenColumnCount = grid.get_allowFrozen() ? grid._defaultFrozenColumnCount : 0; //恢复默认锁定列数
        grid._setFrozenVisible(true);
        grid._internalResetConfig();
        grid._setFrozenVisible(false);
        grid.updateColumnsIcon();
        this._refresh();
        this._setModified();
        this._frozenModified = true;

        grid._rowsTable.style.width = "0px"; //处理Grid在IE中先调节列宽度，然后通过列配置隐藏后再还原列配置时，出现列错位的问题。  2015-05-20  by ljb
        if (!Sys.Browser.isIE || Sys.Browser.version > 8) return;

        asyncCall(function () {
            if (grid.get_readOnly()) {
                grid.refresh(grid.get_dataSource()); //只读情况，直接读取数据集,报表
            } else {
                //处理IE8及以下版本，在同一个Grid上，切换A,B两种列排版时，B排版设置了某个A中没有列的冻结列，然后切换到A中还原配置导致空缺列的问题。
                grid.refresh(grid.saveData(true)); //异步数据刷新.调整为saveData，在单据中用get_DataSource将导致数据被清空
            }
        });
    },

    doDrop: function(sender, args) {
        var grid = this.get_form().grid;
        var dragDatas = grid._dragDatas;
        var sIndex = grid.get_selectedRowIndex();
        args.set_cancel(true); // 自行处理拖动逻辑
        this._toMove(sIndex, dragDatas);
    },

    _toMove: function(sIndex, dragDatas) {
        var grid = this.get_form().grid;
        var selectData = grid.findRowData(sIndex);
        var name = selectData ? selectData.name : null;
        this._setModifyForzen(selectData);
        var columnNames = [];
        for (var i = 0, count = dragDatas.length; i < count; i++) {
            var data = dragDatas[i];
            columnNames.push(data.name);
            this._setModifyForzen(data);
        }
        this.moveColumns(name, columnNames);
        this._refresh(); // 调整完之后，刷新列配置表格

        if (sIndex < 0) sIndex = grid.get_recordCount() - 1;
        setTimeout(function () {
            grid.set_selectedRowIndex(sIndex - 1);
        }, 10);
    },

    _setModifyForzen: function(data) {
        if (!this._frozenModified && data) { //如果移动锁定列，则标记下来
            this._frozenModified = data.lockUrl == this._lockUrl;
        }
    },

    /* 批量移动现有列，比如列配置拖动多列，插入到新的位置 */
    moveColumns: function(columnName, columnNames) {
        var grid = this.get_grid();
        var insertColumn = grid.findColumn(columnName);
        var start, end, destIndex;
        var lastColumn;
        if (insertColumn) {
            start = insertColumn.get_index();
            end = insertColumn.get_index();
            destIndex = insertColumn.get_index();
        } else {
            // 置底功能
            var cls = grid.get_columns();
            lastColumn = cls[cls.length - 1];
            start = lastColumn.get_index();
            end = lastColumn.get_index();
            destIndex = lastColumn.get_index();
        }

        // 计算出影响的列区间
        var newColumns = [];
        for (var i = 0, count = columnNames.length; i < count; i++) {
            var name = columnNames[i];
            var column = grid.findColumn(name);
            newColumns.push(column);

            start = Math.min(start, column.get_index());
            end = Math.max(end, column.get_index());
        }

        // 记录历史column
        var oldColumns = [];
        var insertIndex = 0;
        for (var i = start; i <= end; i++) {
            var column = grid.getColumnByIndex(i);
            if (!Array.contains(newColumns, column)) {
                if (column.get_index() < destIndex) {
                    Array.insert(newColumns, insertIndex, column);
                    insertIndex++;
                } else {
                    newColumns.push(column);
                }
            }
            oldColumns.push(column);
        }

        // 置底
        if (lastColumn) {
            Array.remove(newColumns, lastColumn);
            Array.insert(newColumns, insertIndex, lastColumn);
        }

        // 重定义索引
        for (var i = 0, count = newColumns.length; i < count; i++) {
            var column = newColumns[i];
            column._index = start;
            start++;
        }

        // 重新排序
        var columns = grid._columns;
        columns.sort(function(column1, column2) {
            var index1 = column1.get_index();
            if (typeof (index1) == "undefined") {
                index1 = 0;
            }
            var index2 = column2.get_index();
            if (typeof (index2) == "undefined") {
                index2 = 0;
            }
            return index1 - index2;
        });

        // 记录历史column的elements信息
        var count = oldColumns.length;
        var savedElements = [];
        for (var i = 0; i < count; i++) {
            var column = oldColumns[i];
            savedElements[i] = column._elements;
        }
        // 必须释放对应的列，因为要交换elements
        grid._disposeColumnsHeader(oldColumns);

        // 重新设定column的冻结列索引值
        grid._initColumnForzenIndex();

        // 重新建立关系和刷新数据
        for (var i = 0; i < count; i++) {
            var column = newColumns[i];
            var elements = savedElements[i];
            column._internalSetElements(elements, column.get_index());
            grid._refreshColumn(column);
        }
        grid._setFrozenVisible(false);
        grid._saveColumnsDisplayOrder();
        grid.updateColumnsIcon(newColumns);
        this._setModified();
        if (grid._filterBar) {
            grid._filterBar.reset();
        }
    },

    doDragRow: function(sender) {
        var grid = this.get_form().grid;
        var drag = grid.get_allowDrag();
        if (drag) {
            sender.set_text($craba.getLangPro('启用拖动'));
            grid.set_allowDrag(false);
            grid.set_allowDrop(false);
            grid.set_allowMultiSelectedRow(false);
            grid.set_allowCopy(true);
        } else {
            sender.set_text($craba.getLangPro('禁用拖动'));
            grid.set_allowDrag(true);
            grid.set_allowDrop(true);
            grid.set_allowMultiSelectedRow(true);
            grid.set_allowCopy(false);
        }
    }
}
ApracJs.UI.Grid.GridConfigAction.registerClass('ApracJs.UI.Grid.GridConfigAction', Sys.UI.PageAction);
