/*!
 * apracjs
 * Copyright(c) 2017 by wssf
 * 2017-04-26
 */

'use strict'
var os = require('os');
var utils = require('./utils.js');
var stringUtils = require('./stringUtils.js');

var hiGetter = null;
var inited = false;
var lastLo = null;
var loInc = 1;
var _hi = 0;
var _hi2 = null;
var TimestampBase = (new Date()).getTime();
var MinDateTime = (new Date("2017-06-26 00:00:00")).getTime();
var TickBase = TimestampBase - MinDateTime;

/* 返回数字的cuid */
exports.newCuid = function cuid$newCuid() {
    var lo;
    try {
        if (!inited) {
            init();
        }

        lo = getLo();
        if (lo == lastLo || lo < lastLo + loInc) {
            lo = lastLo + loInc;
            loInc++;
        } else {
            lastLo = lo;
            loInc = 1;
        }
    } catch (ex) {

    } finally {

    }

    var hi = _hi << 48; // 左移6个字节
    return hi + lo;
};

/* 返回字符串的cuid */
exports.newCuidString = function cuid$newCuidString() {
    var lo;
    try {
        if (!inited) {
            init();
        }

        lo = getLo();
        if (lo == lastLo || lo < lastLo + loInc) {
            lo = lastLo + loInc;
            loInc++;
        } else {
            lastLo = lo;
            loInc = 1;
        }
    } catch (ex) {

    } finally {

    }

    return _hi2 + utils.padLeft(lo.toString(), 12, "0");
};

/* 注册cuid的自定义hi接口 */
exports.registerHiGetter = async function(getter) {
    if (hiGetter || !getter || inited) return; // 只初始化一次

    hiGetter = getter;

    console.log('开始初始化cuid的hi');
    if (typeof getter.getHi != 'function') {
        console.log('注册的接口没有实现getHi方法');
        return;
    }

    var siteId = getSiteId();
    var hi = await getter.getHi(siteId);
    if (hi <= 0) {
        console.log('cuid自定义接口生成的hi失败，使用内部生成hi');
        return; // 标记为失败，使用内部自动生成hi，使用newCuidString的情况重复概率也不大；存储空间为2的12次方
    }

    console.log('cuid自定义接口生成的hi为:' + hi);
    makeHi(hi);
    inited = true;
}

function init() {
    inited = true;

    lastLo = getLo();

    makeHashHi();
}

function getLo() {
    var ticks = (new Date()).getTime() - TimestampBase;
    return TickBase + ticks;
}

function makeHashHi() {
    var siteId = getSiteId();
    var hashId = parseInt(utils.getHashCode(siteId));
    makeHi(hashId);
}

function makeHi(hc) {
    _hi = hc & 32; // JavaScript最大整数精度2*53位，时间戳48位，hi剩余5位
    _hi2 = utils.padLeft(hc.toString(), 12, "0");
}

function getSiteId() {
    var siteId = utils.format("{0}_{1}", os.hostname(), process.cwd()); // 机器名称+网站路径
    return siteId;
}

function LoToDateTime(lo) {
    return MinDateTime + lo;
}

function LoToLocalDateTime(lo) {
    return new Date(LoToDateTime(lo));
}