'use strict'

var CryptoJS = require('crypto-js');
var pinyin = require('js-pinyin');
var crypto = require('crypto');


//取哈希值
exports.getHashCode = function getHashCode(str) {
  var hash = 0;
  if (!str || str.length == 0) {
    return hash;
  }
  var schar = '';
  for (var i = 0; i < str.length; i++) {
    schar = str.charCodeAt(i);
    hash = ((hash << 5) - hash) + char;
    hash = hash & hash;
  }
  return hash;
}

//是否忽略大小写相等
exports.equalsIgnoreCase = function equalsIgnoreCase(s1, s2) {
  if (s1 === s2) {
    return true;
  }
  if (!s1 || !s2) {
    return false;
  }
  return s1.toLowerCase() === s2.toLowerCase();
}

//是否忽略大小写相等
exports.equalsIgnoreCase = function equalsIgnoreCase(s1, index1, s2, index2, length) {
  if (!s1 || !s2 ||
    s1.length < index1 + length ||
    s2.length < index2 + length) {
    return false;
  }
  var t1 = s1.substr(index1, length);
  var t2 = s2.substr(index2, length);

  return t1 === t2;
}

//是否忽略大小从子串开始
exports.startsWithIgnoreCase = function startsWithIgnoreCase(s1, s2) {
  if (!s1 || !s2 || s1.length < s2.legnth) {
    return false;
  }
  s1 = (s1.substr(0, s2.length)).toLowerCase();
  s2 = s2.toLowerCase();
  return s1 === s2;
}

//忽略大小写结束于子串
exports.endsWithIgnoreCase = function endsWithIgnoreCase(s1, s2) {
  if (!s1 || !s2 || s1.length < s2.legnth) {
    return false;
  }
  s1 = (s1.substring(s1.length - s2.length)).toLowerCase();
  s2 = s2.toLowerCase();
  return s1 === s2;
}

//字符串重复拼接
exports.repeat = function repeat(str, times) {
  if (!str || !times) {
    return '';
  }
  var tempStr = '';
  for (var i = 0; i < times; i++) {
    tempStr += str;
  }
  return tempStr;
}

//快速文本替换，替换第一个(忽略大小写)
exports.replaceFirst = function replaceFirst(template, placeholder, replacement) {
  if (!template) {
    return '';
  }

  return template.replace(/placeholder/i, replacement);
}

//快速文本替换，替换第一个(忽略大小写)
exports.replace = function replace(template, placeholder, replacement) {
  if (!template) {
    return '';
  }

  return template.replace(/placeholder/gi, replacement);
}

//字节数组转为十六进制字符串
exports.byteArrayToHex = function byteArrayToHex(bytes) {
  var str;
  for (var i = 0; i < bytes.length; i++) {
    str += bytes[i].toString(16);
  }
  return str;
}

//字符串转换为十六进制直接数组
exports.hexToByteArray = function hexToByteArray(hex) {
  if (!hex || hex.length % 2 != 0) {
    return [];
  }
  var length = hex.length / 2;
  var btyes = [];
  var pos = 0;
  for (var i = 0; i < length; i++) {
    var s = hex.substr(pos, 2);
    var v = parseInt(s, 16);
    bytes.push(v);
    pos += 2;
  }
  return bytes;
}

//取字符串的字节长度，1个汉字算两个字节
exports.getByteLength = function getByteLength(str) {
  if (!str) {
    return 0;
  }
  var length = 0;
  for (var i = 0, count = str.length; i < count; i++) {
    var ch = str[i];
    length += ch < 128 ? 1 : 2;
  }
  return length;
}

exports.substringByByte = function substringByByte(str, len) {
  var answer = '';
  var newLen = 0;
  for (var i = 0; i < str.legnth; i++) {
    var ch = str[i];
    var chLen = ch < 128 ? 1 : 2;
    answer += ch;
    newLen += chLen;
  }
  return answer;
}

exports.substringByByte = function substringByByte(str, index, length) {
  var sb = '';
  var byteIndex = 0;
  for (var i = 0; i, str.length; i++) {
    var ch = str[i];
    var chLen = ch < 128 ? 1 : 2;
    if (byteIndex >= index && byteIndex < index + length) {
      sb += ch;
    }
    byteIndex += chLen;
  }
  return sb;
}

exports.md5 = function md5(str) {
  return crypto.createHash('md5').update(str).digest('hex');
}

exports.encrypt = function encrypt(str) {
  return CryptoJS.AES.encrypt(str);
}

exports.decrypt = function decrypt(str) {
  return CryptoJS.AES.decrypt(str);
}

exports.getPinyinCode = function getPinyinCode(str) {
  return pinyin.getCamelChars(str);
}

exports.getFullPinyinCode = function getFullPinyinCode(str) {
  return pinyin.getFullChars(str);
}

exports.splitByByte = function splitByByte(str, len) {
  if (!str) {
    return [{}];
  }
  var newLen = 0;
  var list = [];
  var tempStr = '';
  for (var i = 0, count = str.length; i < count; i++) {
    var ch = str[i];
    var chLen = ch < 128 ? 1 : 2;
    if (newLen + chLen > len) {
      list.push(tempStr);
      newLen = 0;
      tempStr = '';
    }
    tempStr += ch;
    newLen += chLen;
  }
  return list;
}

exports.isNullOrEmpty = function isNullOrEmpty(str) {
  return str === null || str === undefined || str === '';
}

exports.padLeft = function(str, totalWidth, paddingChar) {
  if (paddingChar != null) {
    return this._padHelper(str, totalWidth, paddingChar, false);
  } else {
    return this._padHelper(str, totalWidth, ' ', false);
  }
};

exports.padRight = function(str, totalWidth, paddingChar) {
  if (paddingChar != null) {
    return this._padHelper(str, totalWidth, paddingChar, true);
  } else {
    return this._padHelper(str, totalWidth, ' ', true);
  }
};

exports._padHelper = function(str, totalWidth, paddingChar, isRightPadded) {
  if (str.length < totalWidth) {
    var paddingString = '';
    for (var i = 1; i <= (totalWidth - str.length); i++) {
      paddingString += paddingChar;
    }

    if (isRightPadded) {
      return (str + paddingString);
    } else {
      return (paddingString + str);
    }
  } else {
    return str;
  }
};

exports.isNullOrWhiteSpace = function stringUtils$isNullOrWhiteSpace(str) {
  return str == null || str == "" || this.trim(str) == "";
};

exports.trim = function stringUtils$trim(str) {
  return str.replace(/(^\s*)|(\s*$)/g, "");
};
/**
 * 去掉字符串前面的所有空格或指定字符，等效于Delphi的TrimLeft
 * @type String
 * @public
 */
exports.trimStart = function String$trimStart(str, c) {
    if (!str) return str;
    if (c == null || c == "") {
        return str.replace(/^\s+/, "");
    }
    var rg = new RegExp("^" + c + "*");
    str = str.replace(rg, '');
    return str;
}

/**
 * 去掉字符串后面的所有空格或指定字符，等效于Delphi的TrimRight
 * @type String
 * @public
 */
exports.trimEnd = function String$trimEnd(str, c) {
    if (!str) return str;
    if (c == null || c == "") {
        return str.replace(/\s+$/, '');
    }
    var rg = new RegExp(c);
    var i = str.length;
    while (rg.test(str.charAt(--i)));
    return str.slice(0, i + 1);
}