﻿/*!
 * apracjs
 * Copyright(c) 2017 by wssf
 * 2017-04-26
 */

'use strict';

var fs = require("fs");
var path = require("path");
var iconv = require('iconv-lite');
var crypto = require('crypto');
var stringUtils = require('./stringUtils.js');

exports.defer = typeof setImmediate === 'function' ?
  setImmediate :
  function(fn) {
    process.nextTick(fn.bind.apply(fn, arguments))
  }


exports.isAsyncRequest = function utils$isAsyncRequest(req) {
  var value = req.headers['x-crabaajax'];
  if (!value) return false;
  return value.includes('Delta=true');
}

exports.sessionHash = function(sess) {
  if (!sess) return null;

  // serialize
  var str = JSON.stringify(sess, function(key, val) {
    // ignore sess.cookie property
    if (this === sess &&
      (key === 'cookie' ||
        key === 'reload' ||
        key === 'regenerate' ||
        key === 'destroy' ||
        key === 'touch' ||
        key === 'save')) {
      return;
    }
    return val
  });

  // hash
  return crypto.createHash('sha1').update(str, 'utf8').digest('hex');
}

exports.cloneSession = function(sess) {
  var str = JSON.stringify(sess, function(key, val) {
    // ignore sess.cookie property
    if (this === sess && key === 'cookie') {
      return
    }

    return val
  });

  return JSON.parse(str);
}

exports.filterChar = function(text) {
  //return text.replace(/\s+/g, "").replace(/;/g, "").replace(/\//g, "").replace(/'/g, "").replace(/\)/g, "");
  text = text.replace('_', ''); // 第一个字符不能是下划线
  return text.replace(/[^a-zA-Z0-9_]/g, ''); // 过滤非字母、数字、下划线的其它字符
}

exports.checkError = function utils$checkError(condition, message) {
  // 检查断言并抛出异常
  if (!condition) return;
  if (message) {
    throw new Error("【系统提示】\n\n" + message);
  }
  throw new Error("未知错误");
};

exports.padLeft = function padLeft(text, len, charStr) {
  var s = text + '';
  return new Array(len - s.length + 1).join(charStr + '') + s;
};

exports.padRight = function padRight(text, len, charStr) {
  var s = text + '';
  return new Array(len - s.length + 1).join(charStr + '') + s;
};

exports.format = function utils$format() {
  // 字符串参数化
  var args = new Array(arguments.length);
  for (var i = 0; i < args.length; i++) {
    args[i] = arguments[i];
  }

  if ('string' !== typeof args[0]) {
    args.unshift('{0}');
  }

  var text = args[0];
  text = text.replace(/{-?[0-9]+}/g, function(match, format) {
    var intVal = parseInt(match.substring(1, match.length - 1));
    var replace;
    if (intVal >= 0) {
      replace = args[intVal + 1];
    } else if (intVal === -1) {
      replace = "{";
    } else if (intVal === -2) {
      replace = "}";
    } else {
      replace = "";
    }
    return replace;
  });
  return text;
}

exports.copyTo = function utils$copyTo(from, to) {
  // 对象属性拷贝
  if (to && from) {
    for (var key in from) {
      to[key] = from[key];
    }
  }
  return to;
};

exports.createDelegateForAllFun = function utils$createDelegateForAllFun(from, to) {
  if (to && from) {
    var protos = Object.getPrototypeOf(from);
    var keys = Object.getOwnPropertyNames(protos.__proto__);
    keys.forEach((key) => {
      if (key == "constructor") {
        return;
      }
      to[key] = this.createDelegate(to, from[key]);
    });
    this.copyTo(from, to);
  }
  return to;
}

exports.copyFunTo = function utils$copyFunTo(from, to, instance) {
  // 拷贝一个class的所有方法到另外一个对象
  if (to && from) {
    var keys = Object.getOwnPropertyNames(from);
    for (var i = 0, count = keys.length; i < count; i++) {
      var key = keys[i];
      if (key.startsWith('_') || key == "constructor" || key == 'dispose') continue;
      if (to[key] != undefined) continue;
      to[key] = instance ? this.createDelegate(instance, from[key]) : from[key];
    }
  }
  return to;
};

exports.clone = function clone(obj) {
  //深拷贝一个对象或数组
  if (!obj) return obj;
  return JSON.parse(JSON.stringify(obj));
};

exports.createDelegate = function utils$createDelegate(instance, method) {
  // 创建委托，异步方法使用
  if (typeof (method) == "string") {
    var methodName = method;
    method = instance[methodName];
    if (!method) {
      throw new Error(`事件代码${methodName}不能为空`);
    }
  }

  return function Function$delegate() {
    return method.apply(instance, arguments);
  }
};

exports.lowerFirstChar = function utils$lowerFirstChar(str) {
  // 首字母小写
  var isNotEmpty = typeof str == "string" && str.length > 0;
  return isNotEmpty ? str.substring(0, 1).toLowerCase() + str.substring(1) : "";
};

exports.getUTF8String = function utils$getUTF8String(byte) {
  // 将byte转为utf8编码
  if (byte[0] == 0xef && byte[1] == 0xbb || byte[0] == 0xfe && byte[1] == 0xff || byte[0] == 0xff && byte[1] == 0xfe) {
    return byte.toString(); //utf8 + bom
  } else {
    var newByte = iconv.decode(byte, 'gb2312');
    if (newByte.indexOf('�') > -1) return byte.toString(); // utf8上面没法识别出来，转换后反而会乱码，其实utf8不用转换，直接返回就行了
    return newByte.toString('utf8');
  }
};

/*
不靠谱
exports.isUTF8 = function utils$isUTF8(buff) {
  // 判断字符串是否utf8(有些gbk文件没法判断，弃用)
  for (var i = 0; i < buff.length; i++) {
    if ((buff[i] & 0xE0) == 0xC0) { // 110x xxxx 10xx xxxx
      if ((buff[i + 1] & 0x80) != 0x80) {
        return false;
      }
    } else if ((buff[i] & 0xF0) == 0xE0) { // 1110 xxxx 10xx xxxx 10xx xxxx
      if ((buff[i + 1] & 0x80) != 0x80 || (buff[i + 2] & 0x80) != 0x80) {
        return false;
      }
    } else if ((buff[i] & 0xF8) == 0xF0) { // 1111 0xxx 10xx xxxx 10xx xxxx 10xx xxxx
      if ((buff[i + 1] & 0x80) != 0x80 || (buff[i + 2] & 0x80) != 0x80 || (buff[i + 3] & 0x80) != 0x80) {
        return false;
      }
    }
  }
  return true;
}
// 也不对
exports.isGBK = function utils$isGBK(bytes) {
  if (bytes.length <= 1) {
    return false; // if there is only one byte, it is ASCII code
  } else {
    var byte1 = bytes[0];
    var byte2 = bytes[1];
    if (byte1 >= 129 && byte1 <= 254 && byte2 >= 64 && byte2 <= 254)   //判断是否是GBK编码
    {
      return true;
    }
    else {
      return false;
    }
  }
};*/

exports.getHashCode = function utils$getHashCode(str) {
  // 获取字符串的hash值
  var h = 0;
  var len = str.length;
  var t = 2147483648;
  for (var i = 0; i < len; i++) {
    h = 31 * h + str.charCodeAt(i);
    if (h > 2147483647) h %= t;
  }
  return h.toString();
};

exports.isRequestNotModified = function utils$isRequestNotModified(req, lastModifiedTime) {
  // 检查请求的文件是否更新过
  var ifModifiedSince = 'if-modified-since';
  var modifiedSince = req.headers[ifModifiedSince];
  if (modifiedSince != null) {
    var pos = modifiedSince.indexOf(';');
    if (pos > 0) {
      modifiedSince = modifiedSince.substr(0, pos);
    }
    modifiedSince = new Date(parseInt(modifiedSince));
    if (modifiedSince >= lastModifiedTime) {
      req.res.writeHead(304, "Not Modified");
      return true;
    }
  }
  return false;
};

exports.setCacheLastModified = function utils$setCacheLastModified(res, lastModifiedTime, neverExpires) {
  // 设置文件请求头的最后修改日期
  if (!lastModifiedTime.getTime) {
    lastModifiedTime = new Date(lastModifiedTime);
  }
  res.setHeader("Last-Modified", lastModifiedTime.getTime());
  if (neverExpires) {
    var expires = new Date();
    expires.setYear(expires.getFullYear() + 1);
    res.setHeader("Expires", expires.getTime());
  } else {
    res.setHeader("Expires", -1);
    res.setHeader("Pragma", "no-cache");
  }
};

exports.setCORSHeader = function(res, allowOrigins, allowHeaders, withCredentials, isOPTIONS) {
  // 设置允许跨域
  var origin = res.req.headers["origin"];
  var host = res.req.headers['host'];
  if (origin == undefined || stringUtils.endsWithIgnoreCase(origin, host)) return; // 非跨域请求
  if (allowOrigins.length > 0 && !allowOrigins.includes(origin)) return; // 如果当前请求域名不在运行范围内，不添加跨域请求头

  if (!isOPTIONS) {
    res.setHeader('Access-Control-Expose-Headers', 'requestid,crabarequestid,jsonerror'); // 客户端有读取这类头信息，需要授权否则console窗口很多异常信息：Refused to get unsafe header xxx
  }

  if (allowHeaders == '*') {
    allowHeaders = res.req.headers['access-control-request-headers'] || '';
  }

  res.setHeader('Access-Control-Max-Age', 60 * 60); // 同一url预检查缓存一个小时，不是每个请求都发起预检查 to do
  res.setHeader('Access-Control-Allow-Origin', origin);
  res.setHeader('Access-Control-Allow-Headers', allowHeaders); // 除了默认的头信息外，需要包含客户端Request请求头
  res.setHeader('Access-Control-Allow-Methods', 'POST,GET,OPTIONS');
  if (withCredentials) res.setHeader('Access-Control-Allow-Credentials', true);
};

exports.getFunctionArgs = function(func) {
  // 获取function的参数结构
  var args = func.toString().match(/function\s.*?\(([^)]*)\)/)[1];
  return args.split(",").map(function(arg) {
    return arg.replace(/\/\*.*\*\//, "").trim();
  }).filter(function(arg) {
    return arg;
  });
};

exports.getMethodArgs = function(func) {
  // 获取function的参数结构
  var args = func.toString().match(/.*?\(([^)]*)\)\s*{/)[1];
  return args.split(",").map(function(arg) {
    return arg.replace(/\/\*.*\*\//, "").trim();
  }).filter(function(arg) {
    return arg;
  });
};

exports.mkdirs = function utils$mkdirs(dirname, callback) {
  //递归创建目录 异步方法
  fs.exists(dirname, function(exists) {
    if (exists) {
      callback(true);
    } else {
      this.mkdirs(path.dirname(dirname), function() {
        fs.mkdir(dirname, callback);
      });
    }
  });
};

exports.mkdirsSync = function utils$mkdirsSync(dirname) {
  // 递归创建目录 同步方法
  if (fs.existsSync(dirname)) {
    return true;
  } else {
    if (this.mkdirsSync(path.dirname(dirname))) {
      fs.mkdirSync(dirname);
      return true;
    }
  }
};

exports.pathToArray = function utils$pathToArray(fullPath) {
  if (!fullPath || typeof fullPath != 'string') return [];
  return fullPath.split(path.sep);
};

exports.toWebPath = function utils$toWebPath(fullPath) {
  if (!fullPath) return '';
  return this.pathToArray(fullPath).join('/');
}

exports.getRequestBody = function utils$getRequestBody(req) {
  if (!req.body || Object.keys(req.body).length == 0) return {};
  return JSON.parse(req.body);
}

exports.getIP = function utils$getClientIp(req) {
  if (!req) return '';
  return req.headers['x-forwarded-for'] ||
    req.connection && req.connection.remoteAddress ||
    req.socket && req.socket.remoteAddress ||
    req.connection.socket && req.connection.socket.remoteAddress;
}

exports.makeExceptionHtml = function utils$makeExceptionHtml(err, stringify, req) {
  if (!err) return null;
  if (!err.stack || !err.message) {
    if (err.message) return err.message;
    return stringify === false || typeof err == 'string' ? err : JSON.stringify(err);
  }

  var stack = err.stack.replace(/\n/ig, '<div class="clear"></div>');
  var cwd = process.cwd();
  cwd = cwd.replace(/\\/ig, '\\\\');
  var re = new RegExp(cwd, "ig");
  stack = stack.replace(re, '');

  var config = require('../config.js');
  if (config.apracConfig.env) {
    var Log = require('../log/log4Helper').getInstance();
    Log.writeErr('发起来源:' + this.getIP(req) + ';' + stack); // 正式版本记录异常日志
    return `<div class='ExceptionBox'><div class='ErrMeesge'>${err.message}</div><div style='display:none;'>${stack}</div></div>`; // 正式版输出隐藏堆栈信息
  }

  return `<div class='ExceptionBox'><div class='ErrMeesge'>${err.message}</div>${stack}</div>`
}