﻿/*!
 * 数据库公共静态方法
 * Copyright(c) 2017 by wssf
 * 2017-06-29
 */

'use strict'
var DbHelper = require('./dbHelper.js');

exports.databaseExists = async function(db, database) {
  var sql = "select schema_name from information_schema.schemata where schema_name=:database";
  var result = await db.executeSQL(sql, { database: database });
  return result.values.length > 0;
}

exports.createDatabase = async function(db, database) {
  var sql = `CREATE DATABASE IF NOT EXISTS \`${database}\` DEFAULT CHARACTER SET utf8`;
  var result = await db.executeSQL(sql);
  if (result.values) {
    return result.values.affectedRows > 0;
  } else {
    return result;
  }
}

exports.tableExists = async function(db, database, tablename) {
  var sql = "select TABLE_NAME from information_schema.tables where table_schema=:database and TABLE_NAME=:tablename";
  var result = await db.executeSQL(sql, { database: database, tablename: tablename });
  return result.values.length > 0;
}

exports.createTables = async function(db, sqls) {
  try {
    await db.beginTransaction();

    for (var i = 0; i < sqls.length; i++) {
      var sql = sqls[i];
      await db.executeSQL(sql);
    }

    await db.commitTransaction();
  }
  catch (ex) {
    await db.rollbackTransaction();
  }
}