﻿'use strict'

var multer = require('multer');
var path = require('path');
var fs = require('fs');
var stream = require('stream');
var debug = require('debug')('aprac:WebClientProxy');
var utils = require('../../common/utils.js');
var delegate = utils.createDelegate;
var config = require('../../config.js').apracConfig;
var ajaxHandler;

exports.processRequest = function (result, req, handler) {
  ajaxHandler = handler;
  multerFileUpload(result.value, req, req.res);
}

function multerFileUpload(result, req, res) {
  var dest = '';

  var saveFolder = result.savedDic ? result.savedDic : 'upload';
  if (path.isAbsolute(saveFolder)) { // 绝对路径，存到网站以外的其它目录
    dest = saveFolder;
  } else {
    dest = path.join(config.webPath, saveFolder);  // saveFolder支持相对路径../../../
  }

  var storage = multer.diskStorage({
    destination: dest, //utils.pathToArray(config.webPath + (result.savedDic ? ("\\" + result.savedDic + "\\") : "\\upload\\")).join(path.sep),
    filename: function (req, file, cb) {
      var filename = file.originalname;
      file.extname = path.extname(file.originalname).toLowerCase(); // 扩展名

      try {
        if (result.getFilename) {
          filename = result.getFilename(req, file); // 重定义文件名
        } else {
          var dt = new Date();
          var randNum = Math.random() * 10000;
          file.extname = path.extname(file.originalname).toLowerCase();
          filename = dt.format("yyyyMMddhhmmssS") + randNum + file.originalname; // 默认文件名：日期时间
        }
        file.fileName = filename;
      } catch (ex) {
        return cb(ex);
      }

      try {
        if (result.checkFile) {
          var err = result.checkFile(req, file); // 检查文件类型
          if (err) return cb(err);
        }
      } catch (ex) {
        return cb(ex);
      }
      cb(null, filename);
    }
  });

  var opts = {
    storage: storage
  };
  if (result.fileSize) {
    opts.limits = {
      fileSize: result.fileSize
    };
  }


  var uploader = multer(opts);

  var saveFun;
  if (result.fields && result.fields.length != 0) { //多文件上传
    saveFun = uploader.fields(result.fields);
  } else if (result.filename) { //单文件上传
    saveFun = uploader.single(result.filename);
  } else {
    throw new Error(sysConsts.WebService_MutlerFileUpload_LoseFilename);
  }

  saveFun(req, res, (req, res, (err) => {
    if (err) {
      if (err.code == "LIMIT_FILE_SIZE") {
        err = result.sizeError ? result.sizeError : '文件大小超过了系统允许的范围';
      }
      ajaxHandler.writeExceptionJsonString(req, res, err);
      return;
    }

    try {
      var fileList = [];
      var files = req.files;
      for (var i in files) {
        var fileArr = files[i];
        fileArr.forEach(function (file) {
          delete file.destination; // 安全起见，不要输出服务器路径
          delete file.filename;
          delete file.encoding;
          //file.path = file.path.replace(config.webPath, ''); //这个地址不能替换，因为后面要用这个地址做文件上传和重命名
          fileList.push(file);
        });
      }
      req.files = fileList; // 替换掉默认的

      if (result.doOk) {
        var uploadFinish = result.doOk(err, req, res);
        if (typeof uploadFinish == "function") {
          uploadFinish((data, err) => { // 所有回调完成后，结束页面，并输出json
            if (err) {
              ajaxHandler.writeExceptionJsonString(req, res, err);
            } else {
              ajaxHandler.writeJsonString(req, res, data);
            }
          });
          return;
        }
        ajaxHandler.writeJsonString(req, res, uploadFinish);
      } else {
        ajaxHandler.writeJsonString(req, res, req.files);
      }
    } catch (e) {
      ajaxHandler.writeExceptionJsonString(req, res, e)
    }
  }));
}