﻿/*!
 * apracjs
 * Copyright(c) 2017 by wssf
 * 2017-05-11
 * 生成客户端访问WebMethod方法的代理脚本文件
 */
'use strict'
var fs = require('fs');
var StringBuilder = require('../../common/stringBuilder.js');
var utils = require('../../common/utils.js');
var debug = require('debug')('aprac:WebClientProxy');
var delegate = require('../../common/utils.js').createDelegate;
var WebServiceDataJs = require('./webServiceData.js');
var config = require('../../config.js').apracConfig;

exports.processRequest = function(req, res, done, values) {
  var parsed = req.parsed;
  var filedir = parsed.filedir;
  if (values) {
    var lastModified = values.mtime;
    if (utils.isRequestNotModified(req, lastModified)) {
      res.end();
      return;
    }
    
    var serviceData = values.serviceData;
    var methods = serviceData.methods;
    serviceData.getNames = function() {
      var names = [];
      methods.forEach(element => {
        names.push(element.name);
      });
      return names;
    }
    serviceData.getCount = function() {
      return methods.length;
    }
    serviceData.getMethodArgs = function(mname) {
      var args = {};
      methods.forEach(element => {
        if (mname == element.name) {
          args = element.args;
        }
      });
      return args;
    }
    doClientProxyScript(req, res, lastModified, serviceData);
    return;
  }

  var filename = filedir.replace('.ajax', '.jsx');
  fs.stat(filename, delegate(this, function(err, stat) {
    if (err) {
      debug('文件不存在:%s', filename);
      done();
      return;
    }

    var lastModified = stat.mtime;
    if (utils.isRequestNotModified(req, lastModified)) {
      res.end();
      return;
    }

    doClientProxyScript(req, res, lastModified);
  }));
}

function doClientProxyScript(req, res, lastModified, sData) {
  var serviceData = sData ? sData : WebServiceDataJs.getWebServiceData(req);
  var sb = new StringBuilder();
  sb.appendFormat("Type.registerNamespace('{0}');", serviceData.wnamespace).appendLine();
  sb.appendFormat("{0}=function(){{0}.initializeBase(this);};", serviceData.wnamespace).appendLine();
  sb.appendFormat("{0}.prototype={", serviceData.wnamespace).appendLine();
  doProxyPrototype(sb, serviceData);
  sb.appendFormat("};{0}.registerClass('{0}',Sys.Net.WebServiceProxy);", serviceData.wnamespace).appendLine();
  sb.appendFormat("{0}._staticInstance = new {0}();", serviceData.wnamespace).appendLine();
  sb.appendFormat("{0}._staticInstance.set_path('/{1}.ajax');", serviceData.wnamespace, serviceData.namepath).appendLine();
  sb.append("if (typeof ($app) !== 'undefined') $app.notifyScriptLoaded();").appendLine();

  utils.setCacheLastModified(res, lastModified, config.env); // 产品发布环境才缓存
  res.writeHead(200, { 'Content-Type': 'application/x-javascript; charset=utf-8' });
  res.end(sb.toString());
}

function doProxyPrototype(sb, serviceData) {
  var methodNames = serviceData.getNames();
  for (var i = 0, count = serviceData.getCount(); i < count; i++) {
    if (i > 0) {
      sb.append(",").appendLine();
    }

    var methodName = methodNames[i];
    var paramsName = "";
    var paramsJson = "{";
    var args = serviceData.getMethodArgs(methodName);
    for (var j = 0; j < args.length; j++) {
      var arg = args[j];
      if (j > 0) {
        paramsName += ",";
        paramsJson += ",";
      }
      paramsName += arg;
      paramsJson += arg + ':' + arg;
    }
    paramsJson += "}";
    if (paramsName.length > 0) paramsName += ",";

    sb.appendFormat("{0}: function({1}succeededCallback,failedCallback,userContext){return this._invoke('{0}',{2},succeededCallback,failedCallback,userContext);}",
      methodName, paramsName, paramsJson).appendLine();
  }
}