﻿/*!
 * httpContext
 * Copyright(c) 2017 by wssf
 * 2017-05-02
 */

'use strict'
var utils = require('../../common/utils.js');
var PageInfo = require('./pageinfo.js');
var config = require("../../config.js");

class HttpContext {
  constructor(req, res) {
    req.httpContext = this;
    res.httpContext = this;
    this.req = req;
    this.res = res;
    this.isAsyncRequest = utils.isAsyncRequest(req);

    this._init();
    this.pageInfo = new PageInfo(this);
    this.pageInfo.init();
  }

  dispose() {
    this.req = null;
    this.res = null;
    this.pageInfo.dispose();
    this.pageInfo = null;
  }

  _init() {
    this.clientWidth = this._getCrabaClientSize("ClientWidth=", 984);
    this.clientHeight = this._getCrabaClientSize("ClientHeight=", 555);

    this.pageParams = {};
    var reqBody = this.getReqBody();
    if (reqBody) {
      var params = reqBody['__Params'];
      if (params) {
        this.pageParamsString = params = unescape(params);
        this.pageParams = JSON.parse(params);
      }
    }
  }

  _getCrabaClientSize(key, defaultValue) {
    var headerValues = this.getHeaderValue('x-crabaajax');
    if (!headerValues) return defaultValue;
    headerValues = headerValues.split(',');
    for (var j = headerValues.length - 1; j >= 0; j--) {
      var nameValue = headerValues[j].trim();
      if (nameValue.startsWith(key)) {
        var value = parseInt(nameValue.substr(key.length));
        if (!isNaN(value)) {
          return value;
        }
        break;
      }
    }
    return defaultValue;
  }

  getRequest() {
    return this.req;
  }

  getResponse() {
    return this.rep;
  }

  getIsAsyncRequest() {
    return this.isAsyncRequest;
  }

  getContextDataString() {
    var data = this.getContextData();
    var keys = Object.keys(data);
    for (var i = 0, count = keys.length; i < count; i++) {
      var key = keys[i];
      var value = data[key];
      if (value && typeof value.invalidateCache == "function") {
        delete data[key]; // 没有被使用的数据源object不需要输出到客户端，里面有数据库连接串
      }
    }

    data[config.apracSettings.ClientWidthContextName] = this.clientWidth;
    data[config.apracSettings.ClientHeightContextName] = this.clientHeight;
    data["productmentEnv"] = config.apracConfig.env; // 输出客户端 <Button Visible="${productmentEnv}"

    return JSON.stringify(data);
  }

  getHeaderValue(name) {
    return this.req.headers[name];
  }

  getPageParams() {
    return this.pageParams;
  }

  get_pageParamsString() {
    return this.pageParamsString;
  }

  getReqBody() {
    return this.req.body;
  }

  getQueryValue(key) {
    var query = this.req.parsed.query;
    if (!query) return null;
    if (query[key] == undefined) return null;
    return unescape(query[key]);
  }

  getQueryInt(key) {
    var v = this.getQueryValue(key);
    if (v == null) return 0;
    v = parseInt(v);
    if (isNaN(v)) return 0;
    return v;
  }

  getFormValue(key) {
    var query = this.getReqBody();
    if (!query) return null;
    if (query[key] == undefined) return null;
    return unescape(query[key]);
  }

  getContextData() {
    var data = this.__contextData;
    if (!data || typeof data != "object") {
      data = this.__contextData = {};
    }
    return this.__contextData;
  }
}
module.exports = HttpContext;