﻿/*!
 * httpContext
 * Copyright(c) 2017 by wssf
 * 2017-05-02
 */

'use strict'
var path = require('path');
var utils = require('../../common/utils.js');
var config = require('../../config.js').apracConfig;

class PageInfo {
  constructor(httpContext) {
    this.controller = null;
    this.httpContext = httpContext;
    this.req = httpContext.req;
    this.rootName = "";
    this.rootPath = "";
  }

  dispose() {
    if (this.controller && this.controller.dispose) {
      this.controller.dispose();
      this.controller.req = null;
      this.controller.res = null;
      this.controller.WebMethod = null;
      this.controller.httpContext = null;
      this.controller.user = null;
      this.controller.pageParams = null;
    }
    this.controller = null;
    this.httpContext = null;
    this.req = null;
  }

  init() {
    var req = this.req;
    this.scriptPath = req.parsed.pathname; // "a\b\c\Page.gspx"
    this.parsePathMap();
    if (!this.rootName || this.rootName.length == 0) {
      var name = path.dirname(this.scriptPath);  // a\b\c
      if (name && name !== '.') {
        var namespace = utils.pathToArray(name).join('.'); // a.b.c
        this.rootName = namespace;
      }
    }
    var parsed = req.parsed;
    var filename = parsed.filename;
    var cName = path.basename(filename, '.gspx');
    if (parsed.appName) {
      this.rootName = this.rootName.substr(parsed.appName.length); // 去除分发器路径
    }
    if (this.rootName != "") {
      cName = this.rootName + "." + cName; // a.b.c.Page
    }
    this.controlerPath = cName.replace(/\./g, '/');
    this.controllerName = config.webName + '.' + cName;
  }

  parsePathMap(pathname) {
    var self = this;
    config.pathNames.forEach(function(v, i, a) {
      if (self.scriptPath.startsWith(v)) {
        self.rootName = v;
        self.rootPath = config.pathMap[v];
      }
    });
  }

  setControler(value) {
    this.controller = value;
    if (value != null) {
      this._hasWebService = value.WebMethod != null;
    }
  }

  getScriptVersion() {
    return this.rootName == 'aprac' ? config.versionTag : config.productVersionTag
  }

  hasController() {
    return this.controller != null;
  }

  hasWebService() {
    return !!this._hasWebService;
  }

  webMethodBasePath() {
    return "/" + this.controlerPath + ".ajax";//处理多层文件夹下，访问资源文件时无法下载的问题   by ljb
  }
}

module.exports = PageInfo;
