﻿/*!
 * apracjs
 * Copyright(c) 2017 by wssf
 * 2017-04-26
 * 请求基类，主要处理一些路径和请求头的输出
 */

'use strict'
var path = require('path');
var debug = require('debug')('aprac:handler');
var utils = require('../common/utils.js');
var stringUtils = require('../common/stringUtils.js');
var config = require('../config.js').apracConfig;
var annotationInvoke = require('../apracExtend.js').apracExtendInvoke;

class Handler {
    constructor(app, path, distrributor) {
        this._app = app;
        this._path = path;
        this._distrributor = distrributor;
    }

    init(req, res) {
        req.res = res;
        res.req = req;
        res.setHeader('X-Powered-By', 'ApracJs');
        res.setHeader('X-ApracJs-Author', config.author);
        res.setHeader('X-ApracJs-Version', config.version);
        res.setHeader('X-ApracJs-productmentEnv', config.env);
    }

    enableSession(req) {
        return false;
    }

    getPath() {
        return this._path;
    }

    getAppName() {
        return this._distrributor ? this._distrributor.getAppName() : "";
    }

    parseFileName(parsed) {
        var pathname = parsed.pathname;
        var rootName;
        var rootPath;
        var appName = parsed.appName;
        var subLength = appName.length;
        // if(appName != ""){
        //   subLength = subLength + 1;
        // } 
        pathname = pathname.substr(subLength);
        config.pathNames.forEach(function (v, i, a) {
            if (pathname.startsWith(v)) {
                rootName = v;
                rootPath = config.pathMap[v];
            }
        });

        var lastFilePath;
        if (rootName) {
            parsed.rootPath = rootPath; // 取对应包里面的version需要使用
            lastFilePath = path.join(rootPath, pathname.replace(rootName, ''));
        } else {
            if (appName.length != 0) {
                rootPath = path.join(process.cwd(), config.appFolder, appName);
                lastFilePath = path.join(rootPath, pathname);
            } else {
                rootPath = this._app.get('webPath');
                lastFilePath = path.join(rootPath, pathname);
            }
        }
        return lastFilePath; // 安全问题，只能访问web根路径下面的文件
    }

    parseUrl(req) {
        var parsed = req.parsed;
        if (parsed.filename) return; // 已经解析过

        var pathname = path.normalize(parsed.pathname);
        pathname = pathname.replace(/\.\./g, ""); // 安全问题，禁止大于根路径的父路径请求，只能访问web根路径下面的文件
        var index = pathname.indexOf(`~`);//支持~到根目录访问文件
        if (index >= 0) {
            pathname = pathname.substr(index + 1);
        }
        pathname = stringUtils.trimStart(pathname, '/');
        pathname = stringUtils.trimStart(pathname, '\\\\');
        parsed.pathname = pathname;
        parsed.filename = this.parseFileName(parsed);
        parsed.filedir = path.dirname(parsed.filename);
        //debug("parsedUrl->path:%s ext:%s", parsed.path, parsed.extname);
    }

    processRequest(req, res, done) {
        this.parseUrl(req);
        req.done = done;

        if (this.enableSession(req) && !this._isMemorySession()) { // 内存模式不改写session
            this._bindSession(req, res); // 验证请求开始到结束过程session是否发生改变，如果没有发生改变就不回写（gspx，ajax）
        }
    }

    _waitSessionSaveUnLock(req, callback) {
        var that = this;
        global.setTimeout(function () {
            req.session.reload(function (err) {
                if (err) {
                    callback(err);
                } else if (req.session.__lock && req.session.__expireTimeStamp > new Date().getTime()) {
                    that._waitSessionSaveUnLock(req, callback);
                } else {
                    callback();
                }
            });
        }, 1000);
    }

    _diffSessionField(nowSession, oldSession) {
        let diffFieldInfo = [];
        let filterFields = ["cookie"];
        let nowFidlds = Object.keys(nowSession);
        let oldFidlds = Object.keys(oldSession);

        nowFidlds.forEach(nowFidld => {
            if (filterFields.indexOf(nowFidld) > -1) return;

            let nowValue = nowSession[nowFidld];
            let typeName = typeof nowValue;
            if (typeName === "function") return;

            let exist = false;
            oldFidlds.forEach(oldFidld => {
                if (oldFidld !== nowFidld) return;
                exist = true;
                let oldValue = oldSession[oldFidld];
                if (JSON.stringify(nowValue) !== JSON.stringify(oldValue)) {
                    diffFieldInfo.push({
                        fidld: nowFidld,
                        value: typeName === "object" ? utils.clone(nowValue) : nowValue,
                        action: "update"
                    });
                }
            });

            if (!exist) {
                diffFieldInfo.push({
                    fidld: nowFidld,
                    value: typeName === "object" ? utils.clone(nowValue) : nowValue,
                    action: "add"
                });
            }
        });

        oldFidlds.forEach(oldFidld => {
            if (filterFields.indexOf(oldFidld) > -1) return;

            let oldValue = oldSession[oldFidld];
            if (typeof oldValue === "function") return;

            if (nowFidlds.indexOf(oldFidld) < 0) {
                diffFieldInfo.push({
                    fidld: oldFidld,
                    action: "delete"
                });
            }
        });

        return diffFieldInfo;
    }

    _skipSessionSave(callback, req, renewSessionTimeout) {
        if (renewSessionTimeout && req) {
            //只有在只读情况下续订session过期时间
            req.sessionStore.touch(req.session.id, req.session, function (err) {
                if (callback) utils.defer(callback);
            });
        }
        else {
            if (callback) utils.defer(callback);
        }
    }


    _isMemorySession() {
        return config.sessionConn ? false : true;
    }

    _saveSession(req, skipCallback, diffFieldInfo, sessionSaveFunction, sessionSaveFunctionArgs) {
        console.log("_saveSession======开始保存");
        //保存会话前需要续订Cookie的过期时间，防止无法保存最新的会话
        req.session.touch();

        //保存
        let that = this;
        let expireTime = new Date();
        expireTime.setMinutes(expireTime.getMinutes() + 5);
        req.session.__expireTimeStamp = expireTime.getTime();
        req.session.__lock = true;
        req.session.save(function (err) {
            if (err) {
                console.log("Redis写入错误======" + err.message);
                that._skipSessionSave(skipCallback, req, false);
                return;
            }

            //对比更新差异
            that._patchSession(diffFieldInfo, req.session);

            //解锁
            req.session.__expireTimeStamp = -1;
            req.session.__lock = false;


            //返回到下级方法
            sessionSaveFunction.apply(req.session, sessionSaveFunctionArgs);

            //如果下次再出现Session中的数据未及时同步到Redis就需要用下面这个返回到下级方法
            // //返回到下级方法
            // sessionSaveFunction.call(req.session, function (_err) {
            //     setTimeout(function () {
            //         if (skipCallback) utils.defer(skipCallback, _err);
            //     }, 100);
            // });
        });
        console.log("_saveSession======结束保存");
    }
    _patchSession(diffFieldInfo, session) {
        diffFieldInfo.forEach(fieldInfo => {
            if (fieldInfo.action === "add" ||
                fieldInfo.action === "update") {
                session[fieldInfo.fidld] = fieldInfo.value;
            } else if (fieldInfo.action === "delete") {
                delete session[fieldInfo.fidld];
            }
        });
    }

    _bindSession(req, res) {
        let that = this;
        let _end = res.end;
        let _writeHead = res.writeHead;
        let ended = false;

        //创建副本session，后面需要对比差异
        let _reload = req.session.reload;
        let _regenerate = req.session.regenerate;
        let _destroy = req.session.destroy;
        let _touch = req.session.touch;
        let _save = req.session.save;

        //移除session中的方法，交给平台维护
        req.session.reload = null;
        req.session.regenerate = null;
        req.session.destroy = null;
        req.session.touch = null;
        req.session.save = null;

        //重写
        res.writeHead = function writeHead(statusCode, statusMessage, headers) {
            //还原session方法
            if (req.session.reload === null) {
                req.session.reload = _reload;
                req.session.regenerate = _regenerate;
                req.session.destroy = _destroy;
                req.session.touch = _touch;
                req.session.save = _save;
            }
            _writeHead.call(res, statusCode, statusMessage, headers);
        }

        req.__oldSession = utils.clone(req.session);

        //重写
        res.end = function end(chunk, encoding) {
            console.log(" res.end======res.end");
            if (ended) return false;
            ended = true;

            //校验session是否发生改变，session没有改变的话就不覆写了
            if (utils.sessionHash(req.__oldSession) === utils.sessionHash(req.session)) {
                req.__sessionIsReadOnly = true; //标记为只读，store的set里面判断是写不写
            }

            //还原session方法
            if (req.session.reload === null) {
                req.session.reload = _reload;
                req.session.regenerate = _regenerate;
                req.session.destroy = _destroy;
                req.session.touch = _touch;
                req.session.save = _save;
            }

            req.session.save = function (callback) {
                console.log("req.session.save ======保存");

                //检查session状态
                if (req.__sessionIsReadOnly) {
                    that._skipSessionSave(callback, req, true);
                    return; // 只读的session就不写了
                }

                //对比差异集合
                let diffFieldInfo = that._diffSessionField(req.session, req.__oldSession);

                //reload会改变session，必须通过req.session取得最新的session
                let _args = arguments;
                this.reload(function (err) {
                    console.log(" this.reload======刷新加载" + (err && err.message ? err.message : ""));
                    if (err && err.message !== "failed to load session") {//如果发生错误且不是第一次创建session就不存储
                        that._skipSessionSave(callback, req, false);
                    } else {
                        if (req.session.__lock) {//检查锁
                            that._waitSessionSaveUnLock(req, function (err) {//等待解锁
                                if (err) {
                                    that._skipSessionSave(callback, req, false);
                                    return;
                                }

                                that._saveSession(req, callback, diffFieldInfo, _save, _args);
                            });
                        } else {
                            that._saveSession(req, callback, diffFieldInfo, _save, _args);
                        }
                    }
                });
            }

            //next to express-session
            _end.call(res, chunk, encoding);
        }
    }

    //方法包
    async wrapInterceptor(interceptor, funname, obj, args) {
        try {
            function annotation(desc) {
                var ann = this.__ann;
                if (!ann) {
                    ann = {
                        desc: null,
                        preDesc: '',
                        startTime: new Date().getTime(),
                        detail: {}
                    };
                    this.__ann = ann;
                }
                var end = new Date().getTime();
                ann.endTime = end;
                ann.useTime = `${ann.endTime - ann.startTime}`;
                if (!desc) {
                    return ann;
                }
                if (ann.preDesc) {
                    var item = ann.detail[ann.preDesc];
                    if (item.bindTime) {
                        item.endTime = item.bindTime;
                    } else {
                        item.endTime = end;
                    }
                    item.useTime = `${item.endTime - item.startTime}`;
                }
                ann.preDesc = desc;
                if (ann.detail[desc]) {
                    var item = ann.detail[desc];
                    item.endTime = end;
                    item.useTime = `${item.endTime - item.startTime}`;
                } else {
                    ann.detail[desc] = { startTime: end }
                }
                return ann;
            }
            function releaseAnnotation() {//释放
                var ann = this.__ann;
                if (!ann) {
                    return;
                }
                ann.endTime = new Date().getTime();
                ann.useTime = `${ann.endTime - ann.startTime}`;
                delete ann.preDesc;

                this.annotation = null;
                this.bindAnnotation = null;
                this.releaseAnnotation = null;
            }
            function bindAnnotation(desc, bill) {
                if (!bill) {
                    return;
                }
                var ann = annotation.apply(bill, ['']);
                if (this.__ann && this.__ann.detail) {
                    var preDetail = this.__ann.detail[this.__ann.preDesc];
                    if (preDetail) {
                        preDetail.bindTime = new Date().getTime();
                    }
                    this.__ann.detail[desc] = ann;
                } else {
                    ann.desc = desc;
                }
                bill.annotation = annotation;
                bill.bindAnnotation = bindAnnotation;
                bill.releaseAnnotation = releaseAnnotation;
            }
            var annotationChain = {};
            bindAnnotation.apply(this, [`调用【${interceptor ? interceptor.req.url : funname}】`, annotationChain]);
            if (!interceptor) {
                return await obj[funname].apply(obj, args);
            }

            annotationChain.bindAnnotation(`调用【${funname}】`, obj);
            obj.annotation('业务拦截before');
            await interceptor.before(obj, funname, args);
            obj.annotation('发起业务请求');
            let rt = await obj[funname].apply(obj, args);
            if (typeof (rt) == "function") {//返回值是函数，此时直接把调用嵌套在后面
                obj.annotation('发起业务调用');
                return async (cb) => {
                    var req = interceptor ? interceptor.req : undefined;
                    try {
                        obj.annotation('开始业务处理');
                        await rt(async (result, errormsg) => {
                            obj.annotation('业务处理完毕'); // undefined
                            obj.releaseAnnotation ? obj.releaseAnnotation() : '';
                            if (interceptor) {
                                await interceptor.after(obj, funname, args, result, errormsg);
                            }
                            cb(result, errormsg);
                            releaseAnnotation.apply(annotationChain);
                            annotationInvoke.AnnotationInvoke(req, obj.user, annotationChain);
                        });
                    } catch (e) {
                        obj.releaseAnnotation ? obj.releaseAnnotation() : '';
                        cb(null, e);
                        releaseAnnotation.apply(annotationChain);
                        annotationInvoke.AnnotationInvoke(req, obj.user, annotationChain);
                    }
                }
            } else {
                obj.annotation('业务处理完毕');
            }

            await interceptor.after(obj, funname, args);
            return rt;
        }

        catch (e) {
            if (typeof (e) == "object" || e.message) {
                throw e;
            }
            throw { message: e };
        }
    }
}

module.exports = Handler;
