﻿/*!
 * apracjs
 * Copyright(c) 2017 by wssf
 * 2017-04-26
 * 主要是针对craba3的请求做路由分配，拦截请求处理
 */

'use strict'
var debug = require('debug')('aprac:router');
var url = require('url');
var path = require('path');
var utils = require('../common/utils.js');

class Router {
    constructor(app) {
        this._app = app;
        this._appHandlers = {};//记录路由路径
    }

    addHandler(handler) {
        if (!handler.getPath) {
            throw new TypeError("handler必须包含getPath方法");
        }

        var appName = handler.getAppName().toLowerCase();
        var path = handler.getPath();
        if (!path) {
            throw new TypeError("handler的path不能为空");
        }

        var _handlers = this._appHandlers[appName];
        if (_handlers) {
            if (_handlers[path]) {
                throw new Error(`重复注册appName=${appName}  path=${path}的handler`);
            }
        } else {
            _handlers = [];
            this._appHandlers[appName] = _handlers;
        }

        debug(`成功注册appName=${appName}  path=${path}的handler`);
        _handlers[path] = handler;
        handler.Router = this;
    }

    getHandler(req, path) {
        return this.handle(req, null, null, path);
    }

    _parseUrl(req) {
        var parsed = url.parse(req.url, true);
        var pathname = parsed.pathname;
        var isHome = pathname.slice(-1) === '/'; // 需要先判断，后面转换之后就没有斜杠了
        var pathname = utils.pathToArray(pathname).join(path.sep); // linux和windows统一解决路径问题
        if (isHome) {
          var defaultPage = this._app.get("defaultPage");
          pathname = pathname + utils.pathToArray(defaultPage).join(path.sep);
        }
        parsed.extname = path.extname(pathname).toLowerCase();

        req.parsed = parsed;

        //获取APPName
        var newPathName = parsed.pathname = pathname.slice(1);
        var index = newPathName.indexOf(path.sep);
        if (index > 0) {
            var appName = newPathName.substr(0, index).toLowerCase();
            parsed.relativepath = newPathName.substr(index);
            if (this._appHandlers[appName]) {
                parsed.appName = appName;
            } else {
                parsed.appName = "";
            }
        } else {
            parsed.relativepath = newPathName;
            parsed.appName = "";
        }
    }

    handle(req, res, done, path) {
        this._parseUrl(req);
        var parsed = req.parsed;
        var handlers = this._appHandlers[parsed.appName];
        var handler;
        if (path) {
            return handlers[path];
        }
        if (!parsed.extname) {
            handler = handlers[this._app.get('ajaxTag')];         // 数据服务
        } else {
            handler = handlers[parsed.extname];
            if (!handler) {
                handler = handlers[this._app.get("staticTag")];   // 静态文件
            }
        }
        return handler;
    }
}

module.exports = Router;