﻿var log4js = require('log4js');
var fs = require("fs");
var path = require("path");
var config = require('../config.js').apracConfig;

class log4Helper {
  constructor(logPath, openLog, projectName) {
    //默认开启日志
    if (openLog === false) {
      this.openLog = openLog;
    } else {
      this.openLog = true;
    }

    if (projectName == null) {
      projectName = "apracjs";
    } else {
      projectName = projectName.toLowerCase();
    }

    //[indexname=c_bfune_deploy_d4hist] %date{yyyy-MM-dd HH:mm:ss,fff} [%thread] %level - %message%newline%exception%newline

    //加载配置文件
    var sysPath = '../log4js.json';
    var configPath = config.log4jsonPath;
    if (!configPath) configPath = sysPath; // 使用系统默认配置
    else {
      configPath = path.join(process.cwd(), configPath); // 从产品根路径开始找配置文件
      if (!fs.existsSync(configPath)) configPath = sysPath; // 使用系统默认配置
    }

    if (configPath == sysPath) console.log('log4js使用系统默认配置');
    else console.log('log4js使用产品的配置');

    var objConfig = require(configPath);
    log4js.configure(objConfig);

    this.logDebug = log4js.getLogger('debug');
    this.logInfo = log4js.getLogger('info');
    this.logWarn = log4js.getLogger('warn');
    this.logErr = log4js.getLogger('error');
    console.log('log4js已开启');
  }

  get baseDir() {
    return this._baseDir;
  }

  set baseDir(value) {
    this._baseDir = value;
  }

  get openLog() {
    return this._openLog;
  }

  set openLog(value) {
    this._openLog = value;
  }

  get logDebug() {
    return this._logDebug;
  }

  set logDebug(value) {
    this._logDebug = value;
  }

  get logInfo() {
    return this._logInfo;
  }

  set logInfo(value) {
    this._logInfo = value;
  }

  get logWarn() {
    return this._logWarn;
  }

  set logWarn(value) {
    this._logWarn = value;
  }

  get logErr() {
    return this._logErr;
  }

  set logErr(value) {
    this._logErr = value;
  }

  writeDebug(msg, cb) {
    if (this.openLog) {
      this.logDebug.debug(this.formatLog(msg));
      if (typeof (cb) === 'function') {
        cb(msg);
      }
    }
  }

  writeInfo(msg, cb) {
    if (this.openLog) {
      this.logInfo.info(this.formatLog(msg));
      if (typeof (cb) === 'function') {
        cb(msg);
      }
    }
  }

  writeWarn(msg, cb) {
    if (this.openLog) {
      this.logWarn.warn(this.formatLog(msg));
      if (typeof (cb) === 'function') {
        cb(msg);
      }
    }
  }

  writeErr(msg, cb) {
    if (this.openLog) {
      this.logErr.error(this.formatLog(msg));
      if (typeof (cb) === 'function') {
        cb(msg);
      }
    }
  }

  getTimeStr(time) {
    var timeStr = time.getFullYear() + "-" + (time.getMonth() + 1) + "-" + time.getDate();
    return timeStr;
  }

  formatLog(msg) {
    if (msg === undefined || msg === null)
      msg = "";
    var typeName = typeof (msg);

    switch (typeName) {
      case "string":
        break;
      case "object":
        msg = JSON.stringify(msg);
        break;
      default:
        msg = "";
        break;
    }
    return msg;
  }

  checkAbsPath(baseDir) {
    if (baseDir === undefined || baseDir === null)
      throw new Error("路径不能为空");

    var isAbsPath = false; //是否是绝对路径

    if (path.sep === '\\') { //Windows
      if (baseDir[0] === '\\' || baseDir[0] === '/') {
        throw new Error("Windows系统下配置文件中日志路径不允许以正反斜杠开始！");
      }

      if (baseDir.length > 1 && baseDir[1] === ':') {
        isAbsPath = true;
      }
    } else { //Unix、MacOS、Linux等
      if (baseDir[0] === '\\') {
        throw new Error("Unix、MacOS、Linux等系统下配置文件中日志路径不允许以反斜杠开始！");
      }

      if (baseDir[0] === '/') {
        isAbsPath = true;
      }
    }

    baseDir = baseDir.replace(/\\/g, path.sep).replace(/\//g, path.sep);
    if (baseDir[baseDir.length - 1] != path.sep) {
      baseDir += path.sep;
    }

    return {
      isAbsPath: isAbsPath,
      baseDir: baseDir
    };
  }
}
exports = module.exports = log4Helper;

exports.getInstance = function () {
  if (this._instance) return this._instance;
  return this._instance = new log4Helper(config.LogPath, config.OpenLog, config.webName);
}