﻿var utils = require('../../common/utils.js');
var config = require('../../config.js').apracConfig;

class DbTreeDataSource {
  constructor(data, parentIdFieldName, rootId) {
    this._data = this._getAllParentIdRows(data, parentIdFieldName);
    this._rootId = rootId;
  }

  _getAllParentIdRows(data, parentIdFieldName) {
    var dataArray = {};
    data.forEach(element => {
      var row = element;
      var pId = row[parentIdFieldName];
      var list = dataArray[pId];
      if (!list) {
        list = [];
        dataArray[pId] = list;
      }
      list.push(row);
    });
    return dataArray;
  }

  getNode(textFieldName, idFieldName) {
    var rootTree = {
      ChildNodes: []
    };
    var nodes = this._data[this._rootId];
    if (!nodes) {
      return rootTree;//没有找到需要的
    }
    return this.InitChildNode(rootTree, textFieldName, idFieldName, this._rootId);
  }

  InitChildNode(parentNode, textFieldName, idFieldName, pid) {
    var nodes = this._data[pid];
    delete this._data[pid];
    for (var i in nodes) {
      var node = nodes[i];
      var treeNode = {
        Data: node,
        Text: node[textFieldName],
        ChildNodes: []
      }
      this.InitChildNode(treeNode, textFieldName, idFieldName, node[idFieldName])
      parentNode.ChildNodes.push(treeNode);
    }
    return parentNode;
  }

  get_nodeScriptData(node, root) {
    var answer = [];
    var data = this._getScriptData(node, root);
    if (data) answer.push(data);
    for (var i = 0, count = node.ChildNodes.length; i < count; i++) {
      var child = node.ChildNodes[i];
      answer.push(this.get_nodeScriptData(child));
    }
    return answer;
  }

  _getScriptData(node, root) {
    if (node == root) return null;
    else if (node.get_scriptData) return node.get_scriptData();

    var data = {
      _text: node.Text,
      _data: node.Data
    };
    return data;
  }
}

module.exports = (data, textFieldName, idFieldName, parentIdFieldName, rootId, webMethod) => {
  var tree = new DbTreeDataSource(data, parentIdFieldName, rootId);
  var node = tree.getNode(textFieldName, idFieldName);
  return webMethod ? tree.get_nodeScriptData(node, node) : node;
};